--Libs
AutoLog = LibStub("AceAddon-3.0"):NewAddon("AutoLog", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("AutoLog")
local GetMapNameByID = GetMapNameByID

--map IDs
local map_id_list = {
	["ULD"] = 529, ["NAX"] = 535, ["EYE"] = 527, ["OBS"] = 531, ["VOA"] = 532,
	["TOC"] = 543, ["ONY"] = 718, ["ICC"] = 604, ["RUS"] = 609, ["BWD"] = 754,
	["BTW"] = 758, ["TFW"] = 773, ["BAH"] = 752, ["FIR"] = 800, ["DGS"] = 824,
	["MGS"] = 896, ["TES"] = 886, ["HOF"] = 897
}

--Normal mode raid list
local raid_list = {"ULD", "NAX", "EYE", "OBS", "VOA", "TOC", "ONY", "ICC", "RUS", "BWD", "BTW", "TFW", "BAH", "FIR", "DGS", "MGS", "TES", "HOF"}
--Heroic mode raid list
local raid_heroic = {"TOC", "ICC", "RUS", "BWD", "BTW", "TFW", "FIR", "DGS", "MGS", "TES", "HOF"}
--LFR mode raid lsit
local raid_lfr = {"DGS", "MGS", "TES", "HOF"}
--Option table

local function MakeList(raids)
	local list = {}
	for _, r in ipairs(raids) do list[r] = GetMapNameByID(map_id_list[r]) end
	return list
end

local options = {
	name = "AutoLog",
	handler = AutoLog,
	type = "group",
	args = {
		enable = { -- Global enable key
			type = "toggle",
			name = L["Enable auto logging"],
			desc = L["If unchecked, AutoLog will never enable Combat Logging"],
			get = "IsALenabled",
			set = "ToggleEnable",
			width = "double",
			order = 1,
		},
		dungeons = { -- Heroic dungeons
			type = "toggle",
			name = L["Log 5 player heroic instances"],
			set = "SetDungeons",
			get = "GetDungeons",
			order = 2,
			width = "full",
		},
		--guildRuns = { -- Only log guild runs
		--	type = "toggle",
		--	name = L["Only log guild runs"],
		--	set = "SetGuildRun",
		--	get = "GetGuildRun",
		--	order = 3,
		--	width = "full",
		--},
		curLogging = { -- Logging indicator
			type = "toggle",
			name = L["Logging state"],
			disabled = true,
			desc = L["Checked if combat logging is currently on"],
			get = "IsCurrentlyLogging",
			set = "setCurrLog",
			order = 4,
		},
		raids10 = { -- 10 Normal raids select
			type = "multiselect",
			name = L["10 Players Raids"],
			desc = L["10 Players raid instances where you want to log combat"],
			values = MakeList(raid_list),
			tristate = false,
			set = "SetRaid10",
			get = "GetRaid10",
			order = 5,
		},
		raids25 = { -- 25 Normal raids select
			type = "multiselect",
			name = L["25 Players Raids"],
			desc = L["25 Players raid instances where you want to log combat"],
			values = MakeList(raid_list),
			tristate = false,
			set = "SetRaid25",
			get = "GetRaid25",
			order = 6,
		},
		raids10h = { -- 10 Heroic raids select
			type = "multiselect",
			name = L["10 Players Heroic Raids"],
			desc = L["10 Players heroic raid instances where you want to log combat"],
			values = MakeList(raid_heroic),
			tristate = false,
			set = "SetRaid10h",
			get = "GetRaid10h",
			order = 7,
		},
		raids25h = { -- 25 Heroic raids select
			type = "multiselect",
			name = L["25 Players Heroic Raids"],
			desc = L["25 Players heroic raid instances where you want to log combat"],
			values = MakeList(raid_heroic),
			tristate = false,
			set = "SetRaid25h",
			get = "GetRaid25h",
			order = 8,
		},
		lfr = { -- LFR raids select
			type = "multiselect",
			name = L["LFR Raids"],
			desc = L["Raid finder instances where you want to log combat"],
			values = MakeList(raid_lfr),
			tristate = false,
			set = "SetRaidLFR",
			get = "GetRaidLFR",
			order = 9,
		},
	},
}

--Default DB
local defaults = {
	char = {
		enable = false,
		curLogging = false,
		dungeons = false,
		raids10 = { 
			["ULD"] = false, ["NAX"] = false, ["EYE"] = false, ["OBS"] = false,
			["VOA"] = false, ["TOC"] = false, ["ONY"] = false, ["ICC"] = false,
			["RUS"] = false, ["BWD"] = false, ["BTW"] = false, ["TFW"] = false,
			["BAH"] = false, ["FIR"] = false, ["DGS"] = false, ["MGS"] = false,
			["TES"] = false, ["HOF"] = false,
		},
		raids25 = {
			["ULD"] = false, ["NAX"] = false, ["EYE"] = false, ["OBS"] = false,
			["VOA"] = false, ["TOC"] = false, ["ONY"] = false, ["ICC"] = false,
			["RUS"] = false, ["BWD"] = false, ["BTW"] = false, ["TFW"] = false,
			["BAH"] = false, ["FIR"] = false, ["DGS"] = false, ["MGS"] = false,
			["TES"] = false, ["HOF"] = false,
		},
		raids10h = {
			["TOC"] = false, ["ICC"] = false, ["RUS"] = false, ["BWD"] = false,
			["BTW"] = false, ["TFW"] = false, ["FIR"] = false, ["DGS"] = false,
			["MGS"] = false, ["TES"] = false, ["HOF"] = false, 
		},
		raids25h = {
			["TOC"] = false, ["ICC"] = false, ["RUS"] = false, ["BWD"] = false,
			["BTW"] = false, ["TFW"] = false, ["FIR"] = false, ["DGS"] = false,
			["MGS"] = false, ["TES"] = false, ["HOF"] = false,
		},
		lfr = { ["DGS"] = false, ["MGS"] = false, ["TES"] = false, ["HOF"] = false, }
	},
}

--Init function : Creates DB / chat commands
function AutoLog:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AutoLog", options)
	self.db = LibStub("AceDB-3.0"):New("AutoLog_DB", defaults)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AutoLog", "AutoLog v" .. GetAddOnMetadata("AutoLog", "Version"))
	self:RegisterChatCommand("autolog", "ChatCommand")
	self:RegisterChatCommand("al", "ChatCommand")
end

--Enable function : Register event
function AutoLog:OnEnable()
	self:Print("Version " .. GetAddOnMetadata("AutoLog", "Version") .. L[" loaded."])
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	--if self.db.char.guildrun then self:RegisterEvent("GUILD_PARTY_STATE_UPDATED") end
	self:CheckLog()
end

--Disable function ... should not be called, but must exist
function AutoLog:OnDisable()
	self:Print(L["AutoLog Disabled"])
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	--self:UnregisterEvent("GUILD_PARTY_STATE_UPDATED")
end

--Event handlers
function AutoLog:ZONE_CHANGED_NEW_AREA() self:CheckLog() end
function AutoLog:PLAYER_ENTERING_WORLD() self:CheckLog() end

--[[
function AutoLog:GUILD_PARTY_STATE_UPDATED()
	local  inGroup, numGuildPresent, numGuildRequired, xpMultiplier = InGuildParty()
	print("inGroup",inGroup, "numPresent", numGuildPresent, "numReq", numGuildRequired, "xpMult", xpMultiplier)
end
]]--

--check for Fall of Deathwing component of Dragon Soul
--looks for a raid group in The Maelstrom
local function checkFOD(mid) return ((mid == 751) or (mid == 737)) end

--GetKey function :
--		- Returns the key name of the raid, if it exists, false otherwise
function AutoLog:GetKey()
	local k, v
	local rk = false
	local mid = GetCurrentMapAreaID()
	
	if checkFOD(mid) then rk = "DGS"	
	else
		for k, v in pairs (map_id_list) do
			if v == mid then
				rk = k
				break
			end
		end
	end

	return rk
end

function AutoLog:GetDifficulty() 
	local rd = GetRaidDifficulty()
	if IsInRaid() then 
		if IsPartyLFG() and IsInLFGDungeon() then rd = 5 end
	else
		rd = GetDungeonDifficultyID()
		if rd ~= nil then rd = rd + 10 end		
	end
	return rd
end

--CheckLog function :
--		- Called on event / option changes
--		- Checks whether combat logging should be enabled (or not)
function AutoLog:CheckLog()
	local difficulty = self:GetDifficulty()
	if not difficulty then return end
	local inInstance, instanceType = IsInInstance()
	local isLogging = LoggingCombat()
	local isDungeon = (instanceType == "party")
	local raidKey, raidType
	local goLog = false
	
	if not self.db.char.enable then return end -- Add-on is disabled
	raidKey = self:GetKey()

	if difficulty == 1 then raidType = "raids10"
	elseif difficulty == 2 then raidType = "raids25"
	elseif difficulty == 3 then raidType = "raids10h"; raidKey = self:GetKey(raid_heroic, area)
	elseif difficulty == 4 then raidType = "raids25h"; raidKey = self:GetKey(raid_heroic, area)
	elseif difficulty == 5 then raidType = "lfr"; raidKey = self:GetKey(raid_lfr, area) end

	if ((self.db.char.dungeons) and (isDungeon)) then goLog = true		
	elseif raidKey then goLog = self.db.char[raidType][raidKey] end
	if not inInstance then goLog = false end
	if goLog and not isLogging then self:Print(L["Combat logging turned on"])
	elseif isLogging and not goLog then self:Print(L["Combat logging turned off"]) end

	self.db.char.curLogging = LoggingCombat(goLog)
end

function AutoLog:ChatCommand(input)
	if not input or input:trim() == "" then InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else LibStub("AceConfigCmd-3.0").HandleCommand(AutoLog, "autolog", "AutoLog", input) end
end

function AutoLog:SetCurrLog(info,newValue) self.db.char.curLogging = newValue end

function AutoLog:IsCurrentlyLogging(info)
	self.db.char.curLogging = LoggingCombat()
	return self.db.char.curLogging
end

function AutoLog:IsALenabled(info) return self.db.char.enable end

function AutoLog:ToggleEnable(info, newValue)
	self.db.char.enable = newValue
	self:CheckLog()
end

function AutoLog:GetRaid10(info,raid) return self.db.char.raids10[raid] end

function AutoLog:SetRaid10(info,raid,val)
	self.db.char.raids10[raid] = val
	self:CheckLog()
end

function AutoLog:GetRaid25(info,raid) return self.db.char.raids25[raid] end

function AutoLog:SetRaid25(info,raid,val)
	self.db.char.raids25[raid] = val
	self:CheckLog()
end

function AutoLog:GetRaid10h(info,raid) return self.db.char.raids10h[raid] end

function AutoLog:SetRaid10h(info,raid,val)
	self.db.char.raids10h[raid] = val
	self:CheckLog()
end

function AutoLog:GetRaid25h(info,raid) return self.db.char.raids25h[raid] end

function AutoLog:SetRaid25h(info,raid,val)
	self.db.char.raids25h[raid] = val
	self:CheckLog()
end

function AutoLog:GetDungeons() return self.db.char.dungeons end

function AutoLog:SetDungeons(info, val)
	self.db.char.dungeons = val
	self:CheckLog()
end

function AutoLog:GetRaidLFR(info, raid) return self.db.char.lfr[raid] end

function AutoLog:SetRaidLFR(info, raid, val)
	self.db.char.lfr[raid] = val
	self:CheckLog()
end

--untested code
--[[
function AutoLog:SetGuildRun(info, val)
	self.db.char.guildrun = val
	
	if val then
		self:RegisterEvent("GUILD_PARTY_STATE_UPDATED")
		self:GUILD_PARTY_STATE_UPDATED()
	else
		self:UnregisterEvent("GUILD_PARTY_STATE_UPDATED")
	end
	
	self:CheckLog()
end

function AutoLog:GetGuildRun() return self.db.char.guildrun end
]]--